// numeric_complex.cpp

#include <complex>
#include <iomanip>
#include <iostream>
#include <cmath>

using namespace std;

// compute the complex impedance of a parallel RLC circuit
complex<double> parallel_RLC_impedance( double frequency,
   double resistance, double inductance, double capacitance );

int main( )
{
   const double R = 1000.0;   // R in ohms
   const double L = 0.2;      // L in henries
   const double C = 10.0e-9;  // C in farads
   double frequency;

   // set the resonant frequency
   frequency = 1.0 / sqrt( L * C );

   // compute the impedance at the resonant frequency
   complex<double> impedance =
      parallel_RLC_impedance( frequency, R, L, C );

   cout << setprecision( 2 ) << fixed
      << "AT RESONANT FREQUENCY\nImpedance = "
      << impedance << "   Magnitude = " << abs( impedance )
      << "   Phase = " << arg( impedance ) << endl;

   // compute the impedance at one tenth of the resonant frequency
   impedance = parallel_RLC_impedance( frequency / 10, R, L, C );

   cout << "\nAT ONE TENTH OF RESONANT FREQUENCY\nImpedance = "
      << impedance << "   Magnitude = " << abs( impedance )
      << "   Phase = " << arg( impedance ) << endl;

   // compute the impedance at ten times the resonant frequency
   impedance = parallel_RLC_impedance( frequency * 10, R, L, C );

   cout << "\nAT TEN TIMES THE RESONANT FREQUENCY\nImpedance = "
      << impedance << "   Magnitude = " << abs( impedance )
      << "   Phase = " << arg( impedance ) << endl;
}

inline
complex<double> parallel_RLC_impedance( double frequency,
   double resistance, double inductance, double capacitance )
{
   complex<double> impedance_inverse( 1.0 / resistance,
      frequency * capacitance - 1.0 / ( frequency * inductance ) );

   return 1.0 / impedance_inverse;
}
