// string_whitespace.cpp

#include <algorithm>
#include <cctype>
#include <functional>
#include <iostream>
#include <string>

using namespace std;

void show_stripped( string& s );
void strip_whitespace1( string& s );
void strip_whitespace2( string& s );

int main( )
{
   string front_space( "  Space in front" );
   show_stripped( front_space );

   string back_space( "Space in back\t\t\t " );
   show_stripped( back_space );

   string front_and_back_space( "  Space in front and back\t\t" );
   show_stripped( front_and_back_space );

   string no_space( "ABCDEFGHIJKLMNOPQRSTUVWXYZ" );
   show_stripped( no_space );

   string all_space( "  \n\n\t\t" );
   show_stripped( all_space );
}

void strip_whitespace1( string& s )
{
   // find the first character that is not whitespace
   string::iterator first =
      find_if( s.begin(), s.end(), not1( ptr_fun( isspace ) ) );

   // erase from the beginning to just before that character
   s.erase( s.begin(), first );

   // find the last character that is not whitespace
   string::reverse_iterator last =
      find_if( s.rbegin(), s.rend(), not1( ptr_fun( isspace ) ) );

   // erase from after that character to the end
   s.erase( last.base(), s.end() );
}

void show_stripped( string& s )
{
   cout << "\n\"" << s <<  "\" has " << s.length() << " characters\n";

   strip_whitespace1( s );

   cout << "After stripping, \"" << s <<  "\" has "
      << s.length() << " characters\n";
}

void strip_whitespace2( string& s )
{
   const char whitespace[] = " \n\r\t\v";

   string::size_type spot = s.find_first_not_of( whitespace );

   // if it's all whitespace, delete all characters and return
   if( spot == string::npos )
   {
      s.erase();
      return;
   }

   // if there's leading whitespace, delete it
   else if( spot != 0 )
      s.erase( 0, spot );

   spot = s.find_last_not_of( whitespace );
   if( spot != s.length()-1 )
      s.erase( spot+1 );   // erase to end of string
}
